"*Command.ShapeStyle:			Oval",
"*MenuButton.ShapeStyle:			Oval",
"*MenuButton.leftBitmap:			menu12",
"*translations:				#override \\n\
	<Message>WM_PROTOCOLS:	WMProtocols() \\n",
"*Dialog*value.translations:		#override \\n\
	<Key>Return:		Ok() \\n",
"*xjukebox.label:			Jukebox Controls",
"*jukebox.label:				Jukebox",
"*spoolViewport.width:			300",
"*spoolViewport.height:			100",
"*quitButton.label:			Quit",
"*quitButton.cursorName:			pirate",
"*jukeboxButton.label:			Jukebox",
"*hostButton.label:			Host",
"*aboutButton.label:			About",
"*spoolLabel.label:			Rplay spool",
"*spoolLabel.borderWidth:		0",
"*spoolList*font:			fixed",
"*stopButton.label:			Stop",
"*pauseButton.label:			Pause",
"*continueButton.label:			Continue",
"*soundViewport.width:			200",
"*soundViewport.height:			300",
"*soundLabel.label:			Sounds",
"*soundLabel.borderWidth:		0",
"*soundList*font:			fixed",
"*addButton.label:			Add",
"*addAllButton.label:			Add all",
"*sfileMenuButton.label:			File",
"*serverList.label:			Get list from server",
"*loadList.label:			Load list ...",
"*saveList.label:			Save List ...",
"*queueViewport.width:			200",
"*queueViewport.height:			100",
"*queueLabel.label:			Jukebox",
"*queueLabel.borderWidth:		0",
"*queueList*font:			fixed",
"*playButton.label:			Play",
"*deleteButton.label:			Delete",
"*sortMenuButton.label:			Sort",
"*qfileMenuButton.label:			File",
"*moveToTop.label:			Move to top",
"*moveToBottom.label:			Move to Bottom",
"*sortByName.label:			Sort by name",
"*sortBySize.label:			Sort by size",
"*shuffle.label:				Shuffle",
"*directoryList.label:			Get list from directory ...",
"*spoolViewport.top:			ChainTop",
"*spoolViewport.bottom:			ChainBottom",
"*spoolViewport.left:			ChainLeft",
"*spoolViewport.right:			ChainRight",
"*soundViewport.top:			ChainTop",
"*soundViewport.bottom:			ChainBottom",
"*soundViewport.left:			ChainLeft",
"*soundViewport.right:			ChainRight",
"*queueViewport.top:			ChainTop",
"*queueViewport.bottom:			ChainBottom",
"*queueViewport.left:			ChainLeft",
"*queueViewport.right:			ChainRight",
"*quitButton.top:			ChainTop",
"*quitButton.bottom:			ChainTop",
"*quitButton.left:			ChainLeft",
"*quitButton.right:			ChainLeft",
"*jukeboxButton.fromHoriz:		quitButton",
"*jukeboxButton.top:			ChainTop",
"*jukeboxButton.bottom:			ChainTop",
"*jukeboxButton.left:			ChainLeft",
"*jukeboxButton.right:			ChainLeft",
"*hostButton.fromHoriz:			jukeboxButton",
"*hostButton.top:			ChainTop",
"*hostButton.bottom:			ChainTop",
"*hostButton.left:			ChainLeft",
"*hostButton.right:			ChainLeft",
"*aboutButton.fromHoriz:			hostButton",
"*aboutButton.top:			ChainTop",
"*aboutButton.bottom:			ChainTop",
"*aboutButton.left:			ChainLeft",
"*aboutButton.right:			ChainLeft",
"*spoolLabel.fromVert:			quitButton",
"*spoolLabel.top:			ChainTop",
"*spoolLabel.bottom:			ChainTop",
"*spoolLabel.left:			ChainLeft",
"*spoolLabel.right:			ChainLeft",
"*spoolViewport.fromVert:		spoolLabel",
"*spoolViewport.allowVert:		True",
"*spoolList.forceColumns:		True",
"*spoolList.defaultColumns:		1",
"*spoolList.horizDistance:		0",
"*stopButton.fromVert:			spoolViewport",
"*stopButton.top:			ChainBottom",
"*stopButton.bottom:			ChainBottom",
"*stopButton.left:			ChainLeft",
"*stopButton.right:			ChainLeft",
"*pauseButton.fromVert:			spoolViewport",
"*pauseButton.fromHoriz:			stopButton",
"*pauseButton.top:			ChainBottom",
"*pauseButton.bottom:			ChainBottom",
"*pauseButton.left:			ChainLeft",
"*pauseButton.right:			ChainLeft",
"*continueButton.fromVert:		spoolViewport",
"*continueButton.fromHoriz:		pauseButton",
"*continueButton.top:			ChainBottom",
"*continueButton.bottom:			ChainBottom",
"*continueButton.left:			ChainLeft",
"*continueButton.right:			ChainLeft",
"*soundLabel.top:			ChainTop",
"*soundLabel.bottom:			ChainTop",
"*soundLabel.left:			ChainLeft",
"*soundLabel.right:			ChainLeft",
"*soundViewport.fromVert:		soundLabel",
"*soundViewport.allowVert:		True",
"*soundList.forceColumns:		True",
"*soundList.defaultColumns:		1",
"*soundList.horizDistance:		0",
"*addButton.fromVert:			soundViewport",
"*addButton.top:				ChainBottom",
"*addButton.bottom:			ChainBottom",
"*addButton.left:			ChainLeft",
"*addButton.right:			ChainLeft",
"*addAllButton.fromVert:			soundViewport",
"*addAllButton.fromHoriz:		addButton",
"*addAllButton.top:			ChainBottom",
"*addAllButton.bottom:			ChainBottom",
"*addAllButton.left:			ChainLeft",
"*addAllButton.right:			ChainLeft",
"*sfileMenuButton.fromVert:		soundViewport",
"*sfileMenuButton.fromHoriz:		addAllButton",
"*sfileMenuButton.top:			ChainBottom",
"*sfileMenuButton.bottom:		ChainBottom",
"*sfileMenuButton.left:			ChainLeft",
"*sfileMenuButton.right:			ChainLeft",
"*queueLabel.top:			ChainTop",
"*queueLabel.bottom:			ChainTop",
"*queueLabel.left:			ChainLeft",
"*queueLabel.right:			ChainLeft",
"*queueViewport.fromVert:		queueLabel",
"*queueViewport.allowVert:		True",
"*queueList.forceColumns:		True",
"*queueList.defaultColumns:		1",
"*queueList.horizDistance:		0",
"*playButton.fromVert:			queueViewport",
"*playButton.top:			ChainBottom",
"*playButton.bottom:			ChainBottom",
"*playButton.left:			ChainLeft",
"*playButton.right:			ChainLeft",
"*deleteButton.fromVert:			queueViewport",
"*deleteButton.fromHoriz:		playButton",
"*deleteButton.top:			ChainBottom",
"*deleteButton.bottom:			ChainBottom",
"*deleteButton.left:			ChainLeft",
"*deleteButton.right:			ChainLeft",
"*sortMenuButton.fromVert:		queueViewport",
"*sortMenuButton.fromHoriz:		deleteButton",
"*sortMenuButton.top:			ChainBottom",
"*sortMenuButton.bottom:			ChainBottom",
"*sortMenuButton.left:			ChainLeft",
"*sortMenuButton.right:			ChainLeft",
"*qfileMenuButton.fromVert:		queueViewport",
"*qfileMenuButton.fromHoriz:		sortMenuButton",
"*qfileMenuButton.top:			ChainBottom",
"*qfileMenuButton.bottom:		ChainBottom",
"*qfileMenuButton.left:			ChainLeft",
"*qfileMenuButton.right:			ChainLeft",
