#ifndef	rplaytool_HEADER
#define	rplaytool_HEADER

/*
 * rplaytool_ui.h - User interface object and function declarations.
 * This file was generated by `gxv' from `rplaytool.G'.
 * DO NOT EDIT BY HAND.
 */

extern Attr_attribute	INSTANCE;

extern Xv_opaque	rplaytool_file_menu_create();
extern Xv_opaque	rplaytool_cont_menu_create();
extern Xv_opaque	rplaytool_pause_menu_create();
extern Xv_opaque	rplaytool_stop_menu_create();
extern Xv_opaque	rplaytool_delete_menu_create();
extern Xv_opaque	rplaytool_add_menu_create();
extern Xv_opaque	rplaytool_playlistfile_menu_create();
extern Xv_opaque	rplaytool_move_menu_create();

typedef struct {
	Xv_opaque	w;
	Xv_opaque	c;
	Xv_opaque	file;
	Xv_opaque	host;
	Xv_opaque	volume;
	Xv_opaque	spool;
	Xv_opaque	cont;
	Xv_opaque	pause;
	Xv_opaque	stop;
} rplaytool_w_objects;

extern rplaytool_w_objects	*rplaytool_w_objects_initialize();

extern Xv_opaque	rplaytool_w_w_create();
extern Xv_opaque	rplaytool_w_c_create();
extern Xv_opaque	rplaytool_w_file_create();
extern Xv_opaque	rplaytool_w_host_create();
extern Xv_opaque	rplaytool_w_volume_create();
extern Xv_opaque	rplaytool_w_spool_create();
extern Xv_opaque	rplaytool_w_cont_create();
extern Xv_opaque	rplaytool_w_pause_create();
extern Xv_opaque	rplaytool_w_stop_create();

typedef struct {
	Xv_opaque	s;
	Xv_opaque	controls7;
	Xv_opaque	directory;
	Xv_opaque	disklist;
	Xv_opaque	controls9;
	Xv_opaque	soundlist;
	Xv_opaque	controls1;
	Xv_opaque	add;
	Xv_opaque	close;
	Xv_opaque	controls4;
	Xv_opaque	playlist3;
	Xv_opaque	controls5;
	Xv_opaque	playlist4;
	Xv_opaque	controls2;
	Xv_opaque	playlist1;
	Xv_opaque	controls3;
	Xv_opaque	playlist2;
	Xv_opaque	controls6;
	Xv_opaque	move;
	Xv_opaque	delete;
	Xv_opaque	file;
	Xv_opaque	playit;
	Xv_opaque	next;
	Xv_opaque	prev;
	Xv_opaque	stopit;
	Xv_opaque	whichlist;
	Xv_opaque	controls8;
	Xv_opaque	sort;
	Xv_opaque	reload;
	Xv_opaque	soundsource;
} rplaytool_s_objects;

extern rplaytool_s_objects	*rplaytool_s_objects_initialize();

extern Xv_opaque	rplaytool_s_s_create();
extern Xv_opaque	rplaytool_s_controls7_create();
extern Xv_opaque	rplaytool_s_directory_create();
extern Xv_opaque	rplaytool_s_disklist_create();
extern Xv_opaque	rplaytool_s_controls9_create();
extern Xv_opaque	rplaytool_s_soundlist_create();
extern Xv_opaque	rplaytool_s_controls1_create();
extern Xv_opaque	rplaytool_s_add_create();
extern Xv_opaque	rplaytool_s_close_create();
extern Xv_opaque	rplaytool_s_controls4_create();
extern Xv_opaque	rplaytool_s_playlist3_create();
extern Xv_opaque	rplaytool_s_controls5_create();
extern Xv_opaque	rplaytool_s_playlist4_create();
extern Xv_opaque	rplaytool_s_controls2_create();
extern Xv_opaque	rplaytool_s_playlist1_create();
extern Xv_opaque	rplaytool_s_controls3_create();
extern Xv_opaque	rplaytool_s_playlist2_create();
extern Xv_opaque	rplaytool_s_controls6_create();
extern Xv_opaque	rplaytool_s_move_create();
extern Xv_opaque	rplaytool_s_delete_create();
extern Xv_opaque	rplaytool_s_file_create();
extern Xv_opaque	rplaytool_s_playit_create();
extern Xv_opaque	rplaytool_s_next_create();
extern Xv_opaque	rplaytool_s_prev_create();
extern Xv_opaque	rplaytool_s_stopit_create();
extern Xv_opaque	rplaytool_s_whichlist_create();
extern Xv_opaque	rplaytool_s_controls8_create();
extern Xv_opaque	rplaytool_s_sort_create();
extern Xv_opaque	rplaytool_s_reload_create();
extern Xv_opaque	rplaytool_s_soundsource_create();

typedef struct {
	Xv_opaque	f;
	Xv_opaque	controls10;
	Xv_opaque	filedir;
	Xv_opaque	filelist;
	Xv_opaque	filename;
	Xv_opaque	filedoit;
	Xv_opaque	fileclose;
} rplaytool_f_objects;

extern rplaytool_f_objects	*rplaytool_f_objects_initialize();

extern Xv_opaque	rplaytool_f_f_create();
extern Xv_opaque	rplaytool_f_controls10_create();
extern Xv_opaque	rplaytool_f_filedir_create();
extern Xv_opaque	rplaytool_f_filelist_create();
extern Xv_opaque	rplaytool_f_filename_create();
extern Xv_opaque	rplaytool_f_filedoit_create();
extern Xv_opaque	rplaytool_f_fileclose_create();
#endif
